#ifndef H_CV_MULTIWIN
#define H_CV_MULTIWIN

#include <highgui.h>
#include <cv.h>
#include <stdio.h>


//	CvMultiWin	
//  Adrian Kaehler
//
//  10/10/2004, AK:  Initial release for use by Stanford Racing Team
//
// This is the CvMultiWin Class.  It is intended for helping in debugging activities
// with OpenCV.  The basic idea is that a CvMultiWin is loosely analogous to a window
// generated by cvNamedWindow, but you can put multiple "panels" into it do display
// mutiple things, instead of having a desktop full of individual windows.
//
//  To create a multi-window, you should first create an instance of the CvMultiWin
// object.  When using it for debugging, it is best to make it global, so many routines
// can write to it without having to pass pointers to the CvMultiWin object all over
// the place.
// 
//  Once the object has been craeted, it must be initialized.  Initialization
// requires the size of the image panels to be set (they are all the
// same size!), and the number of images to be set (number of rows and
// number of columns).  The color depth and number of channels are also
// set at initialization time.
//
//   When the CvMultiWin object has been initialized, it can be written to
// with the command CvMultiWin::paint().  Paint requires at least an 
// IplImage* pointer for the image to copy into the CvMultiWin.  Additionally,
// a single integer may be passed, which will be interpreted as a
// pannel number (scanning left to right, top to bottom).  If two numbers are
// provided, they will be interpreted as column, row indices for the panel
// to write to.  Finally, the fourth argument can be used to tell the
// object to only update the memory image and not actually redraw the
// CvMultiWin.  This is useful if you wish to write to many panels before
// actually updating the window.  The window will be drawn to either when
// a paint request is sent with draw_now=true, or an explicit call to
// CvMultiWin::redraw() is made.
//
//   If any doubt exists as to whether a CvMultiWin is initialized, a
// utility membwer CvMultiWin::initialized() will return a boolean to 
// indicate if it has been or it has not been.
//
//   This code should be understood to be developmental, and may contain bugs.
// Users are invited to make any changes they see fit, and requested to
// make modified or debugged versions generally available to all interested
// parties.  Please maintain a change log in the headder.
//
//   Permission is granted to copy, distribute, or modify this code in any
// way and for any purpose they may see fit so long as the author's original
// name remains intact in this headder.
//
class CvMultiWin {

private:

	char		m_szWindowName[256];
	int			m_width,		m_height;
	int			m_panel_width,	m_panel_height;
	int			m_rows;
	int			m_columns;
	IplImage*	m_image;
	int			m_implicit_target;
	bool		m_initialized;
	char**		m_labels;

	static bool		M_CLASS_INITIALIZED;
	static int		M_SPC;
	static CvFont*	M_FONT;

public:

	// Default constructor, basically does nothing.  (Note that a window
	// is NOT created on the screen until CvMultiWin::initialize() is
	// called.
	//
	CvMultiWin() {
		m_image = NULL;
		m_initialized=false;
	}

	// Get rid of the window and free the memory needed for the
	// multi-panel image.
	//
	~CvMultiWin() {
		if( m_image != NULL ) cvReleaseImage( &m_image );
		cvDestroyWindow( m_szWindowName );
	}

	// The user is required to make an initialize() call before using
	// the multi-window.  This call should indicate the name for the
	// window (as per cvNamedWindow()), as well as the width and height
	// of the panel images.  c,r specify the number of columns and
	// rows (respectively) of panels to create.  depth and channels
	// are passed directly to the multi-panel image.
	//
	void initialize( 
		const char* name, 
		int w, 
		int h, 
		int c=2, 
		int r=2, 
		int depth=8, 
		int channels=3 ,
		int flags=0
	);

	void setLabel( int c, int r, char* txt ) {
		strncpy( m_labels[r*m_columns+c], txt, 256 );
	}

	// Here the caller gives an image and we will draw it into multiwindow.
	// If the caller omits the parameter "r", then the argument "rc" is
	// interpreted as the position in raster scan order.  If "r" is supplied,
	// then rc is interpreted as the column, and r is the row.
	// If 'rc' is not provided either, then the image is drawn to m_implicit_target,
	// which is then incremented.  m_implicit_target is cleared on a draw
	// command.
	//
	// TODO: it would be nice if we checked to make sure the user was
	// drawing to a legal panel...
	//
	void paint( IplImage* in, int rc=-1, int r=-1, bool redraw_now=false );

	// Sopmetimes someone wants to know if *this is initialized.  Usually this
	// is to avoid initializing twice!
	//
	bool initialized() { return( m_initialized ); }

	void redraw() {

		if( ! m_initialized ) return;
		
		if( m_image != NULL ) {
			m_implicit_target=0;
			cvShowImage( m_szWindowName, m_image );
		}
	
	}


};









#endif

